/********************************************
 * UTIA AV CR, v.v.i
 * Signal Processing Department
 * Pod Vodarenskou vezi 4
 * 182 08 Prague
 * Czech Republic
 * web: http://zs.utia.cas.cz
 * contact: sp@utia.cz
 *
 * Filename: adv7611_im.c
 * Date: 15.09.2016
 * Author: Lukas Kohout <kohoutl@utia.cas.cz>
 * Description:
 *    Avnet FMC HDMI IN (ADV7611) interface
 *
 * Changelog:
 *
 *
 *
 *
 ********************************************/

#include <stdio.h>
#include <string.h>
#include <math.h>
#include <unistd.h>

// #include <xil_io.h>
//
// #include <xiic.h>
// #include <xiic_l.h>
//
// #include "xparameters.h"
// #include <sleep.h>

#include "adv7611_im.h"

#include "../i2c/i2c_im.h"


#define INTERLACE_SUPPORT   0
// #define INTERLACE_SUPPORT   1

//***********************************************************************************

/*
 * ADV7611 address space mapping configuration
 */
const adv7611_im_i2c_t adv7611_im_mapping_conf[] =
{
    {'w', I2C_ADV7611_IM_BASE_ADDR >> 1, 0xF4, I2C_ADV7611_IM_CEC_ADDR        },  // CEC
    {'w', I2C_ADV7611_IM_BASE_ADDR >> 1, 0xF5, I2C_ADV7611_IM_INFOFRAME_ADDR  },  // INFOFRAME
    {'w', I2C_ADV7611_IM_BASE_ADDR >> 1, 0xF8, I2C_ADV7611_IM_DPLL_ADDR       },  // DPLL
    {'w', I2C_ADV7611_IM_BASE_ADDR >> 1, 0xF9, I2C_ADV7611_IM_KSV_ADDR        },  // KSV
    {'w', I2C_ADV7611_IM_BASE_ADDR >> 1, 0xFA, I2C_ADV7611_IM_EDID_ADDR       },  // EDID
    {'w', I2C_ADV7611_IM_BASE_ADDR >> 1, 0xFB, I2C_ADV7611_IM_HDMI_ADDR       },  // HDMI
    {'w', I2C_ADV7611_IM_BASE_ADDR >> 1, 0xFD, I2C_ADV7611_IM_CP_ADDR         },  // CP
    {0,0,0,0}
};


/*
 * ADV7611 EDID - configuration befor EDID will be written
 */
const adv7611_im_i2c_t adv7611_im_edid_pre_conf[] =
{
    {'w', I2C_ADV7611_IM_KSV_ADDR >> 1, 0x77, 0x00},    // Disable the Internal EDID
    {0,0,0,0}
};


/*
 * ADV7611 EDID post configuration
 */
const adv7611_im_i2c_t adv7611_im_edid_post_conf[] =
{
    {'w', I2C_ADV7611_IM_KSV_ADDR >> 1, 0x77, 0x00},    // Set the Most Significant Bit of the SPA location to 0
    {'w', I2C_ADV7611_IM_KSV_ADDR >> 1, 0x52, 0x20},    // Set the SPA for port B.
    {'w', I2C_ADV7611_IM_KSV_ADDR >> 1, 0x53, 0x00},    // Set the SPA for port B.
    {'w', I2C_ADV7611_IM_KSV_ADDR >> 1, 0x70, 0x9E},    // Set the Least Significant Byte of the SPA location
    {'w', I2C_ADV7611_IM_KSV_ADDR >> 1, 0x74, 0x03},    // Enable the Internal EDID for Ports
    {0,0,0,0}
};


/*
 * Hot plug assert configuration
 */
const adv7611_im_i2c_t adv7611_im_set_hpa_conf[] =
{
    {'w', I2C_ADV7611_IM_HDMI_ADDR >> 1, 0x6C, 0xA4},   // HPA_DELAY_SEL = 0b1010 1s delay
                                                        // HPA_AUTO_INT_EDID = 0b10 active EDID and cable detect
                                                        // HPA_MANUAL=0  disabled
    {0,0,0,0}
};



/*
 * ADV7611 main configuration
 */
// #define MANUAL_CP
const adv7611_im_i2c_t adv7611_im_init_conf[] =
{
    {'w', I2C_ADV7611_IM_BASE_ADDR  >> 1, 0x01, 0x06},  // Prim_Mode=0b0110 HDMI-GR
    {'w', I2C_ADV7611_IM_BASE_ADDR  >> 1, 0x02, 0xF0},  // Auto CSC, YCrCb out, Set op_656_range to full, disable ALT_DATA_SAT
//     {'w', I2C_ADV7611_IM_BASE_ADDR  >> 1, 0x02, 0x00},  // Auto CSC, YCrCb out, Set op_656_range to full, disable ALT_DATA_SAT
    {'w', I2C_ADV7611_IM_BASE_ADDR  >> 1, 0x03, 0x80},  // 16-Bit SDR ITU-R BT.656 4:2:2 Mode 0
    {'w', I2C_ADV7611_IM_BASE_ADDR  >> 1, 0x04, 0x62},  // OP_CH_SEL[2:0] = 011b - (P[15:8] Y, P[7:0] CrCb), XTAL_FREQ[1:0] = 01b (28.63636 MHz)
    {'w', I2C_ADV7611_IM_BASE_ADDR  >> 1, 0x05, 0x2C},  // AV Codes on

    {'w', I2C_ADV7611_IM_CP_ADDR    >> 1, 0x7b, 0x05},  //

#ifdef MANUAL_CP
    //CP CSC
    {'w', I2C_ADV7611_IM_CP_ADDR  >> 1, 0x69, 0x10}, // MAN_CP_CSC_EN
    {'w', I2C_ADV7611_IM_CP_ADDR  >> 1, 0x68, 0x00}, // CSC_COEFF_SEL[3:0]=[7:4]=0000=manual

    //******************************************************************
    {'w', I2C_ADV7611_IM_CP_ADDR  >> 1, 0x58, 0x64}, // A1 = 0x58[7:0]
    {'w', I2C_ADV7611_IM_CP_ADDR  >> 1, 0x57, 0x29}, // A1 = 0x57[4:0]
    {'w', I2C_ADV7611_IM_CP_ADDR  >> 1, 0x56, 0x99}, // A2 = 0x56[7:0]
    {'w', I2C_ADV7611_IM_CP_ADDR  >> 1, 0x55, 0x4C}, // A2 = 0x55[1:0]
    {'w', I2C_ADV7611_IM_CP_ADDR  >> 1, 0x54, 0x07}, // A3 = 0x57[6:0]

    //******************************************************************
    {'w', I2C_ADV7611_IM_CP_ADDR  >> 1, 0x5F, 0x4D}, // B1 = 0x58[7:0]
    {'w', I2C_ADV7611_IM_CP_ADDR  >> 1, 0x5E, 0x19}, // B1 = 0x57[4:0]
    {'w', I2C_ADV7611_IM_CP_ADDR  >> 1, 0x5D, 0x00}, // B2 = 0x56[7:0]
    {'w', I2C_ADV7611_IM_CP_ADDR  >> 1, 0x5C, 0xCD}, // B2 = 0x55[1:0]
    {'w', I2C_ADV7611_IM_CP_ADDR  >> 1, 0x5B, 0x7A}, // B3 = 0x57[6:0]

    //******************************************************************
    {'w', I2C_ADV7611_IM_CP_ADDR  >> 1, 0x66, 0xB3}, // C1 = 0x58[7:0]
    {'w', I2C_ADV7611_IM_CP_ADDR  >> 1, 0x65, 0xBA}, // C1 = 0x57[4:0]
    {'w', I2C_ADV7611_IM_CP_ADDR  >> 1, 0x64, 0xA9}, // C2 = 0x56[7:0]
    {'w', I2C_ADV7611_IM_CP_ADDR  >> 1, 0x63, 0x03}, // C2 = 0x55[1:0]
    {'w', I2C_ADV7611_IM_CP_ADDR  >> 1, 0x62, 0x20}, // C3 = 0x57[6:0]

    {'w', I2C_ADV7611_IM_CP_ADDR  >> 1, 0x52, 0x00}, //

    {'w', I2C_ADV7611_IM_CP_ADDR  >> 1, 0x59, 0x08}, //
    {'w', I2C_ADV7611_IM_CP_ADDR  >> 1, 0x5A, 0x00}, //

    {'w', I2C_ADV7611_IM_CP_ADDR  >> 1, 0x60, 0x08}, //
    {'w', I2C_ADV7611_IM_CP_ADDR  >> 1, 0x61, 0x00}, //

    //manual offset
    {'w', I2C_ADV7611_IM_CP_ADDR  >> 1, 0x77, 0xC1}, //
    {'w', I2C_ADV7611_IM_CP_ADDR  >> 1, 0x78, 0xF0}, //
    {'w', I2C_ADV7611_IM_CP_ADDR  >> 1, 0x79, 0x7C}, //
    {'w', I2C_ADV7611_IM_CP_ADDR  >> 1, 0x7A, 0x1F}, //

    // manual gain
    {'w', I2C_ADV7611_IM_CP_ADDR  >> 1, 0x73, 0xD0}, //
    {'w', I2C_ADV7611_IM_CP_ADDR  >> 1, 0x74, 0x94}, //
    {'w', I2C_ADV7611_IM_CP_ADDR  >> 1, 0x75, 0x25}, //
    {'w', I2C_ADV7611_IM_CP_ADDR  >> 1, 0x76, 0x09}, //

    {'w', I2C_ADV7611_IM_CP_ADDR  >> 1, 0x3E, 0x00}, // VID_ADJ_EN
    {'w', I2C_ADV7611_IM_CP_ADDR  >> 1, 0x3A, 0x00}, // CP_CONTRAST[7:0]
    {'w', I2C_ADV7611_IM_CP_ADDR  >> 1, 0x3B, 0x00}, // CP_SATURATION[7:0],
#endif

    {'w', I2C_ADV7611_IM_BASE_ADDR  >> 1, 0x0B, 0x44}, // Power up part
    {'w', I2C_ADV7611_IM_BASE_ADDR  >> 1, 0x0C, 0x42}, // Power up part
    {'w', I2C_ADV7611_IM_BASE_ADDR  >> 1, 0x14, 0x7F}, // Max Drive Strength
    {'w', I2C_ADV7611_IM_BASE_ADDR  >> 1, 0x15, 0x80}, // Disable Tristate of Pins
    {'w', I2C_ADV7611_IM_BASE_ADDR  >> 1, 0x06, 0xA1}, // LLC polarity (INV_LLC_POL = 1)
    {'w', I2C_ADV7611_IM_BASE_ADDR  >> 1, 0x19, 0x80}, // LLC DLL phase (delay = 0)
    {'w', I2C_ADV7611_IM_BASE_ADDR  >> 1, 0x33, 0x40}, // LLC DLL enable

    {'w', I2C_ADV7611_IM_CP_ADDR    >> 1, 0xBA, 0x01}, // Set HDMI FreeRun
    {'w', I2C_ADV7611_IM_CP_ADDR    >> 1, 0x6C, 0x00}, //

    {'w', I2C_ADV7611_IM_KSV_ADDR   >> 1, 0x40, 0x81}, // Disable HDCP 1.1 features

    {'w', I2C_ADV7611_IM_HDMI_ADDR  >> 1, 0x9B, 0x03}, // ADI recommended setting
    {'w', I2C_ADV7611_IM_HDMI_ADDR  >> 1, 0x6F, 0x08}, // ADI recommended setting (optimized DVI detection)
    {'w', I2C_ADV7611_IM_HDMI_ADDR  >> 1, 0x85, 0x1F}, // ADI recommended setting
    {'w', I2C_ADV7611_IM_HDMI_ADDR  >> 1, 0x87, 0x70}, // ADI recommended setting
    {'w', I2C_ADV7611_IM_HDMI_ADDR  >> 1, 0x57, 0xDA}, // ADI recommended setting
    {'w', I2C_ADV7611_IM_HDMI_ADDR  >> 1, 0x58, 0x01}, // ADI recommended setting

    {'w', I2C_ADV7611_IM_HDMI_ADDR  >> 1, 0x03, 0x98}, // ADI recommended setting --
    {'w', I2C_ADV7611_IM_HDMI_ADDR  >> 1, 0x4C, 0x44}, // ADI recommended setting --

    {'w', I2C_ADV7611_IM_HDMI_ADDR  >> 1, 0xC1, 0x01}, // ADI recommended setting
    {'w', I2C_ADV7611_IM_HDMI_ADDR  >> 1, 0xC2, 0x01}, // ADI recommended setting
    {'w', I2C_ADV7611_IM_HDMI_ADDR  >> 1, 0xC3, 0x01}, // ADI recommended setting
    {'w', I2C_ADV7611_IM_HDMI_ADDR  >> 1, 0xC4, 0x01}, // ADI recommended setting
    {'w', I2C_ADV7611_IM_HDMI_ADDR  >> 1, 0xC5, 0x01}, // ADI recommended setting
    {'w', I2C_ADV7611_IM_HDMI_ADDR  >> 1, 0xC6, 0x01}, // ADI recommended setting
    {'w', I2C_ADV7611_IM_HDMI_ADDR  >> 1, 0xC7, 0x01}, // ADI recommended setting
    {'w', I2C_ADV7611_IM_HDMI_ADDR  >> 1, 0xC8, 0x01}, // ADI recommended setting
    {'w', I2C_ADV7611_IM_HDMI_ADDR  >> 1, 0xC9, 0x01}, // ADI recommended setting
    {'w', I2C_ADV7611_IM_HDMI_ADDR  >> 1, 0xCA, 0x01}, // ADI recommended setting
    {'w', I2C_ADV7611_IM_HDMI_ADDR  >> 1, 0xCB, 0x01}, // ADI recommended setting
    {'w', I2C_ADV7611_IM_HDMI_ADDR  >> 1, 0xCC, 0x01}, // ADI recommended setting

    {'w', I2C_ADV7611_IM_HDMI_ADDR  >> 1, 0x00, 0x08}, // Set HDMI Input Port A  (BG_MEAS_PORT_SEL = 001b)
    {'w', I2C_ADV7611_IM_HDMI_ADDR  >> 1, 0x02, 0x03}, // Enable Ports A & B in background mode
    {'w', I2C_ADV7611_IM_HDMI_ADDR  >> 1, 0x83, 0xFC}, // Enable clock terminators for port A & B
    {'w', I2C_ADV7611_IM_HDMI_ADDR  >> 1, 0x8D, 0x04}, // LFG Port A
    {'w', I2C_ADV7611_IM_HDMI_ADDR  >> 1, 0x8E, 0x1E}, // HFG Port A
    {'w', I2C_ADV7611_IM_HDMI_ADDR  >> 1, 0x1A, 0x8A}, // Unmute audio
    {'w', I2C_ADV7611_IM_HDMI_ADDR  >> 1, 0x75, 0x10}, // DDC drive strength
    {'w', I2C_ADV7611_IM_HDMI_ADDR  >> 1, 0x90, 0x04}, // LFG Port B
    {'w', I2C_ADV7611_IM_HDMI_ADDR  >> 1, 0x91, 0x1E}, // HFG Port B

    {'w', I2C_ADV7611_IM_HDMI_ADDR  >> 1, 0x47, 0x02}, // QZERO_RGB_FULL

    {'w', I2C_ADV7611_IM_HDMI_ADDR  >> 1, 0x56, 0x7F}, // Enable 5V filter, set maximal time ~ 6us

    {0,0,0,0}
};


/*
 * ADV7611 timing regs
 */
const adv7611_im_i2c_t adv7611_im_timing_regs[] =
{
    {'r', I2C_ADV7611_IM_HDMI_ADDR >> 1, 0x05, 0},
    {'r', I2C_ADV7611_IM_HDMI_ADDR >> 1, 0x07, 0},
    {'r', I2C_ADV7611_IM_HDMI_ADDR >> 1, 0x08, 0},
    {'r', I2C_ADV7611_IM_HDMI_ADDR >> 1, 0x09, 0},
    {'r', I2C_ADV7611_IM_HDMI_ADDR >> 1, 0x0A, 0},
    {'r', I2C_ADV7611_IM_HDMI_ADDR >> 1, 0x0B, 0},
    {'r', I2C_ADV7611_IM_HDMI_ADDR >> 1, 0x20, 0},
    {'r', I2C_ADV7611_IM_HDMI_ADDR >> 1, 0x21, 0},
    {'r', I2C_ADV7611_IM_HDMI_ADDR >> 1, 0x22, 0},
    {'r', I2C_ADV7611_IM_HDMI_ADDR >> 1, 0x23, 0},
    {'r', I2C_ADV7611_IM_HDMI_ADDR >> 1, 0x24, 0},
    {'r', I2C_ADV7611_IM_HDMI_ADDR >> 1, 0x25, 0},
    {'r', I2C_ADV7611_IM_HDMI_ADDR >> 1, 0x2A, 0},
    {'r', I2C_ADV7611_IM_HDMI_ADDR >> 1, 0x2B, 0},
    {'r', I2C_ADV7611_IM_HDMI_ADDR >> 1, 0x2E, 0},
    {'r', I2C_ADV7611_IM_HDMI_ADDR >> 1, 0x2F, 0},
    {'r', I2C_ADV7611_IM_HDMI_ADDR >> 1, 0x32, 0},
    {'r', I2C_ADV7611_IM_HDMI_ADDR >> 1, 0x33, 0},
    {'r', I2C_ADV7611_IM_HDMI_ADDR >> 1, 0x51, 0},
    {'r', I2C_ADV7611_IM_HDMI_ADDR >> 1, 0x52, 0},
    {0,0,0,0}
};


const adv7611_im_i2c_t adv7611_im_timing_interleaced_regs[] =
{
    {'r', I2C_ADV7611_IM_HDMI_ADDR >> 1, 0x0C, 0},
    {'r', I2C_ADV7611_IM_HDMI_ADDR >> 1, 0x28, 0},
    {'r', I2C_ADV7611_IM_HDMI_ADDR >> 1, 0x29, 0},
    {'r', I2C_ADV7611_IM_HDMI_ADDR >> 1, 0x2C, 0},
    {'r', I2C_ADV7611_IM_HDMI_ADDR >> 1, 0x2D, 0},
    {'r', I2C_ADV7611_IM_HDMI_ADDR >> 1, 0x30, 0},
    {'r', I2C_ADV7611_IM_HDMI_ADDR >> 1, 0x31, 0},
    {'r', I2C_ADV7611_IM_HDMI_ADDR >> 1, 0x34, 0},
    {'r', I2C_ADV7611_IM_HDMI_ADDR >> 1, 0x35, 0},
    {0,0,0,0}
};


// ***************************************************************************



/*
 *  checks whether HDMI IN cable is plugged or not - detect 5V
 */
int adv7611_im_cable_detect(int i2c_fd)
{
    int status;

    status = adv7611_im_i2c_sel(i2c_fd, I2C_ADV7611_IM_MUX_DATA);
    if (status < 0) return -1;

    status = adv7611_im_i2c_read(i2c_fd, I2C_ADV7611_IM_BASE_ADDR >> 1, ADV7611_IM_REG_VS_INF_PB_0_28);

    if (status < 0) {
         printf("ERROR: Failed to read from I2C!\n");
        return -2;
    }
    //return status & ADV7611_IM_CABLE_DET_A_RAW;
    return ( (status & ADV7611_IM_CABLE_DET_A_RAW) == (ADV7611_IM_CABLE_DET_A_RAW) ) ? 1 : 0;
}


/*
 * write ADV7611 configuration by I2C
 */
int adv7611_im_init_sequence(int i2c_fd, const adv7611_im_i2c_t conf[])
{
    unsigned int i = 0;
    int status;

    status = adv7611_im_i2c_sel(i2c_fd, I2C_ADV7611_IM_MUX_DATA);
    if (status < 0) return -1;

    while(conf[i].rw) {
        switch(conf[i].rw) {
            case 'r':
                status = adv7611_im_i2c_read(i2c_fd, conf[i].space, conf[i].reg);
                if (status < 0) return -2;
                break;
            case 'w':
                status = adv7611_im_i2c_write(i2c_fd, conf[i].space, conf[i].reg, conf[i].data);
                if (status < 0) return -3;
                break;
            default:
                break;
        }
        i++;
    }
    return 0;
}


/*
 * write the contents of the HDMI input's EDID EEPROM.
 */
int adv7611_im_write_edid(int i2c_fd, unsigned char edid[256] )
{
    int status;
    int i;

    /*
     * pre config
     */
    status = adv7611_im_init_sequence(i2c_fd, adv7611_im_edid_pre_conf);
    if (status < 0) {
        return -1;
    }

    /*
     * write the EDID
     */
    for (i = 0; i < 256; i++) {
        status = adv7611_im_i2c_write(i2c_fd, I2C_ADV7611_IM_EDID_ADDR >> 1, i, edid[i]);
        if (status < 0) return -2;
    }

    /*
     * post config
     */
    status = adv7611_im_init_sequence(i2c_fd, adv7611_im_edid_post_conf);
    if (status < 0) {
        return -3;
    }

    return 0;
}


/*
 * get lock on incomming video signal
 */
int adv7611_im_get_lock(int i2c_fd)
{
    int status = 0;

    status = adv7611_im_i2c_sel(i2c_fd, I2C_ADV7611_IM_MUX_DATA);
    if (status < 0) return -1;

    status |= adv7611_im_i2c_read(i2c_fd, I2C_ADV7611_IM_BASE_ADDR >> 1, ADV7611_IM_REG_IO_6A);
    if (status < 0) return -2;

    status |= adv7611_im_i2c_read(i2c_fd, I2C_ADV7611_IM_HDMI_ADDR >> 1, ADV7611_IM_REG_HDMI_07);
    if (status < 0) return -3;

    // Get lock status based on
    //    ADV7611-HDMI[0x04][1] = TMDS PLL lock
    //    ADV7611-HDMI[0x07][5] = de regen filter lock
    //    ADV7611-HDMI[0x07][7] = vertical filter lock
    return ( (status & ADV7611_IM_LOCKED) == (ADV7611_IM_LOCKED) ) ? 1 : 0;
}


/*
 * set hpd of ADV7611 on the AVNET FMC board
 */
int adv7611_im_set_hpd(int i2c_fd, unsigned char hpd)
{
    int status;
    unsigned char reg_data;

    status = adv7611_im_i2c_sel(i2c_fd, I2C_IO_EXP_IM_MUX_DATA);
    if (status < 0) return -1;

    status = adv7611_im_i2c_read(i2c_fd, I2C_IO_EXP_IM_BASE_ADDR, 0x01);
    if (status < 0) return -2;

    if (hpd)  reg_data = (unsigned char)status |  0x04;
    else      reg_data = (unsigned char)status & ~0x04;

    status = adv7611_im_i2c_write(i2c_fd, I2C_IO_EXP_IM_BASE_ADDR, 0x01, reg_data);
    if (status < 0) return -3;

    return 0;
}


/*
 * reset ADV7611
 */
int adv7611_im_rst(int i2c_fd)
{
    int status;
    unsigned char reg_data;

    status = adv7611_im_i2c_sel(i2c_fd, I2C_IO_EXP_IM_MUX_DATA);
    if (status < 0) return -1;

    status = adv7611_im_i2c_read(i2c_fd, I2C_IO_EXP_IM_BASE_ADDR, 0x01);
    if (status < 0) return -2;

    reg_data = (unsigned char)status & ~ADV7611_IM_RSTN;

    status = adv7611_im_i2c_write(i2c_fd, I2C_IO_EXP_IM_BASE_ADDR, 0x01, reg_data);
    if (status < 0) return -3;

    // 5 ms pulse is minimum
    usleep(10000);

    reg_data |= ADV7611_IM_RSTN;

    status = adv7611_im_i2c_write(i2c_fd, I2C_IO_EXP_IM_BASE_ADDR, 0x01, reg_data);
    if (status < 0) return -4;

    return 0;
}


/*
 * Get a timing of currently locked video signal
 */
int adv7611_im_get_timing(int i2c_fd, adv7611_im_timing_t *tm)
{
    int i;
    int status;
    int timeout;
    unsigned char reg_data[256];

    unsigned int  is_hdmi;
    unsigned int  is_encrypted;
    unsigned int  is_interlaced;
    unsigned int  color_depth;

    // horizontal timing
    unsigned int  h_active_video;
    unsigned int  h_front_porch;
    unsigned int  h_sync_width;
    unsigned int  h_back_porch;
    unsigned int  h_sync_polarity;
    //unsigned int h_total_width;

    // vertical timing progresive or interlaced field0
    unsigned int  v0_active_video;
    unsigned int  v0_front_porch;
    unsigned int  v0_sync_width;
    unsigned int  v0_back_porch;
    //unsigned int v0_total_height;

#if (INTERLACE_SUPPORT == 1)
    // vertical timing interlaced field1
    unsigned int  v1_active_video;
    unsigned int  v1_front_porch;
    unsigned int  v1_sync_width;
    unsigned int  v1_back_porch;
    unsigned int  v1_total_height;
#endif

    // common vertical sync polarity
    unsigned int  v_sync_polarity;

    // pixel clock
    unsigned int  tmds_freq;
    unsigned int  tmds_freq_frac;

    status = adv7611_im_i2c_sel(i2c_fd, I2C_ADV7611_IM_MUX_DATA);
    if (status < 0) return -1;


    // wait on locked timing
    timeout = 100;
    if ( !tm->locked ) {
        while ( !tm->locked && timeout-- ) {
            tm->locked = adv7611_im_get_lock( i2c_fd );
            if ( !tm->locked ) usleep(100000); // wait 100 ms before another try
        }
        if ( !tm->locked ) {
            //printf( "\nHDMII: NOT locked on incoming video!\n");
            return -2;
        }
        usleep(100000); // wait for 100msec to stabilize the timing
    }

    // get timing registers
    i = 0;
    while(adv7611_im_timing_regs[i].rw) {
        status = adv7611_im_i2c_read(i2c_fd, adv7611_im_timing_regs[i].space, adv7611_im_timing_regs[i].reg);
        if (status < 0) return -3;
        reg_data[adv7611_im_timing_regs[i].reg] = (unsigned char)status;
        i++;
    }

#if (INTERLACE_SUPPORT == 1)
    i = 0;
    while(adv7611_im_timing_interleaced_regs[i].rw) {
        status = adv7611_im_i2c_read(i2c_fd, adv7611_im_timing_interleaced_regs[i].space, adv7611_im_timing_interleaced_regs[i].reg);
        if (status < 0) return -4;
        reg_data[adv7611_im_timing_interleaced_regs[i].reg] = (unsigned char)status;
        i++;
    }
#endif

    // get general input video info
    is_hdmi         = (reg_data[0x05] >> 7) & 0x01;
    is_encrypted    = (reg_data[0x05] >> 6) & 0x01;
    is_interlaced   = (reg_data[0x0B] >> 5) & 0x01;
    color_depth     = (reg_data[0x0B] >> 6) & 0x03;

    // get horizontal input video info
    h_active_video  = ((unsigned int)(reg_data[0x07] & 0x1F) << 8) | ((unsigned int)reg_data[0x08]);
    h_front_porch   = ((unsigned int)(reg_data[0x20] & 0x1F) << 8) | ((unsigned int)reg_data[0x21]);
    h_sync_width    = ((unsigned int)(reg_data[0x22] & 0x1F) << 8) | ((unsigned int)reg_data[0x23]);
    h_back_porch    = ((unsigned int)(reg_data[0x24] & 0x1F) << 8) | ((unsigned int)reg_data[0x25]);
    h_sync_polarity = (reg_data[0x05] >> 5) & 0x01;
    //h_total_width   = ((unsigned int)(reg_data[0x1E] & 0x3F) << 8) | ((unsigned int)reg_data[0x1F]);

    // get horizontal input video info - prograssive or interlaced fieald 0
    v0_active_video = ((unsigned int)(reg_data[0x09] & 0x1F) << 8) | ((unsigned int)reg_data[0x0A]);
    v0_front_porch  = ((unsigned int)(reg_data[0x2A] & 0x3F) << 8) | ((unsigned int)reg_data[0x2B]);
    v0_sync_width   = ((unsigned int)(reg_data[0x2E] & 0x3F) << 8) | ((unsigned int)reg_data[0x2F]);
    v0_back_porch   = ((unsigned int)(reg_data[0x32] & 0x3F) << 8) | ((unsigned int)reg_data[0x33]);
    //v0_total_height = ((unsigned int)(reg_data[0x26] & 0x3F) << 8) | ((unsigned int)reg_data[0x27]);

#if (INTERLACE_SUPPORT == 1)
    v1_active_video = ((unsigned int)(reg_data[0x0B] & 0x1F) << 8) | ((unsigned int)reg_data[0x0C]);
    v1_front_porch  = ((unsigned int)(reg_data[0x2C] & 0x3F) << 8) | ((unsigned int)reg_data[0x2D]);
    v1_sync_width   = ((unsigned int)(reg_data[0x30] & 0x3F) << 8) | ((unsigned int)reg_data[0x31]);
    v1_back_porch   = ((unsigned int)(reg_data[0x34] & 0x3F) << 8) | ((unsigned int)reg_data[0x35]);
    v1_total_height = ((unsigned int)(reg_data[0x28] & 0x3F) << 8) | ((unsigned int)reg_data[0x29]);
#endif

    v_sync_polarity = (reg_data[0x05] >> 4) & 0x01;

    //get TMDS freq - pixel clock
    tmds_freq      = ((unsigned int)(reg_data[0x51] & 0xFF) << 1) | ((unsigned int)(reg_data[0x52] & 0x80) >> 7);
    tmds_freq_frac = (unsigned int)(reg_data[0x52] & 0x7F);

    // initialize timing structure
    memset( tm, 0x00, sizeof(adv7611_im_timing_t) );

    tm->is_hdmi       = is_hdmi;
    tm->is_encrypted  = is_encrypted;
    tm->is_interlaced = is_interlaced;
    tm->color_depth   = (color_depth == 3) ? 16 : ( 8 + (color_depth * 2) );

    tm->hav           = h_active_video;
    tm->hfp           = h_front_porch;
    tm->hsw           = h_sync_width;
    tm->hbp           = h_back_porch;
    tm->hsp           = h_sync_polarity;

    if ( is_interlaced ) {
      // TBD
    } else { // Progressive
        tm->vav = v0_active_video;
        tm->vfp = v0_front_porch >> 1;
        tm->vsw = v0_sync_width  >> 1;
        tm->vbp = v0_back_porch  >> 1;
        tm->vsp = v_sync_polarity;
    }

    tm->tmds_freq = (float)(tmds_freq)+(float)tmds_freq_frac/128.0;

    return 0;
}


/*
 *  print input timing
 */
void adv7611_im_print_timing(adv7611_im_timing_t *tm)
{
    double        int_part;
    unsigned int  line_len;
    unsigned int  line_cnt;
    float         fps;

    printf( "ADV7611 Video Input Information\n" );
    printf( "\tVideo Input      = %s", tm->is_hdmi ? "HDMI" : "DVI" );
    printf( "%s", tm->is_encrypted ? ", HDCP Encrypted" : "" );
    printf( ", %s\n", tm->is_interlaced ? "Interlaced" : "Progressive" );
    printf( "\tColor Depth      = %d bits per channel\n", tm->color_depth );
    printf( "\tHSYNC Timing     = hav=%04d, hfp=%02d, hsw=%02d(hsp=%d), hbp=%03d\n",
                tm->hav,
                tm->hfp,
                tm->hsw,
                tm->hsp,
                tm->hbp
    );
    printf( "\tVSYNC Timing     = vav=%04d, vfp=%02d, vsw=%02d(vsp=%d), vbp=%03d\n",
                tm->vav,
                tm->vfp,
                tm->vsw,
                tm->vsp,
                tm->vbp
    );
    printf( "\tVideo Dimensions = %d x %d\n", tm->hav, tm->vav );

    printf( "\tPixel Clock      = %d.%d MHz\n",  (unsigned int)floor(tm->tmds_freq), (unsigned int)(modf((double)tm->tmds_freq, &int_part)*1000000.0));

    line_len = tm->hav + tm->hfp + tm->hsw + tm->hbp;
    line_cnt = tm->vav + tm->vfp + tm->vsw + tm->vbp;

    fps = (tm->tmds_freq * 1000000.0) / ((float)line_len * (float)line_cnt);

    printf( "\tFrame rate       = %d.%d FPS\n",(unsigned int)floor(fps), (unsigned int)(modf((double)fps, &int_part)*1000000.0));
}


/*
 * initialize ADV7611 HDMI
 * it is required the I2C controlloer has already been initialized
 */
int adv7611_im_init(int i2c_fd, unsigned char edid[256])
{
    int status = 0;

    /*
     * reset codec
     */
    status = adv7611_im_rst(i2c_fd);
    if (status < 0) {
        printf("Cannot reset ADV7611 ... ");
        return -1;
    }
    usleep(100000);

    /*
     * adress spaces maping
     */
    status = adv7611_im_init_sequence(i2c_fd, adv7611_im_mapping_conf);
    if (status < 0) {
        printf("Cannot map ADV7611 ... ");
        return -2;
    }

    /*
     * set HPA
     */
    status = adv7611_im_init_sequence(i2c_fd, adv7611_im_set_hpa_conf);
    if (status < 0) {
        printf("Cannot set HPA ... ");
        return -3;
    }

    /*
     * write EDID
     */
    status = adv7611_im_write_edid(i2c_fd, edid);
    if (status < 0) {
        printf("Cannot write EDID ... ");
        return -4;
    }

    /*
     * set HPD
     */
    status = adv7611_im_set_hpd(i2c_fd, 1);
    if (status < 0) {
        printf("Cannot set HPD ... ");
        return -5;
    }
    usleep(100000);

    /*
     * ADV7611 main config
     */
    status = adv7611_im_init_sequence(i2c_fd, adv7611_im_init_conf);
    if (status < 0) {
        printf("Cannot init the codec ... ");
        return -6;
    }
    usleep(100000);


    return 0;
}


