/********************************************
 * UTIA AV CR, v.v.i
 * Signal Processing Department
 * Pod Vodarenskou vezi 4
 * 182 08 Prague
 * Czech Republic
 * web: http://zs.utia.cas.cz
 * contact: sp@utia.cz
 *
 * Filename: vtc.h
 *
 * Date: 2022.03.07
 * Author: Lukas Kohout <kohoutl@utia.cas.cz>
 * Description:
 *    Implements functions for handling Video Timing Controller
 *
 * Changelog:
 *
 *
 *
 *
 ********************************************/

#ifndef _VTC_HEADER_FILE_
#define _VTC_HEADER_FILE_

#ifdef __cplusplus
extern "C" {
#endif


#include <stdint.h>
#include <assert.h>
#include <dirent.h>
#include <fcntl.h>
#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <sys/mman.h>
#include <unistd.h>
#include <stddef.h>


#include "xvtc.h"



#define XVTC_ERR_A_CHR_LOCK_MASK    0x00200000
#define XVTC_ERR_A_VID_LOCK_MASK    0x00100000
#define XVTC_ERR_HSYNC_LOCK_MASK    0x00080000
#define XVTC_ERR_VSYNC_LOCK_MASK    0x00040000
#define XVTC_ERR_HBLANK_LOCK_MASK   0x00020000
#define XVTC_ERR_VBLANK_LOCK_MASK   0x00010000
#define XVTX_ERR_ALL_MASK           ( XVTC_ERR_A_CHR_LOCK_MASK  | \
                                      XVTC_ERR_A_VID_LOCK_MASK  | \
                                      XVTC_ERR_HSYNC_LOCK_MASK  | \
                                      XVTC_ERR_VSYNC_LOCK_MASK  | \
                                      XVTC_ERR_HBLANK_LOCK_MASK | \
                                      XVTC_ERR_VBLANK_LOCK_MASK ) \


#define VTC_SEL_NONE      255 // not selected
#define VTC_1920x1080p60  0
#define VTC_1920x1200p50  1
#define VTC_1600x1200p50  2
#define VTC_1680x1050p60  3
#define VTC_1280x1024p60  4
#define VTC_1280x720p60   5
#define VTC_1024x768p60   6
#define VTC_800x600p60    7
#define VTC_640x480p60    8
#define VTC_600x800p60    9


#define VTC_RES_CNT     10

struct struct_vtc_timing_t
{
    char      *name;

    uint32_t  vav;    // vertical active video
    uint32_t  vfp;    // vertical front porch
    uint32_t  vsw;    // vertical sync width
    uint32_t  vbp;    // vertical back porch
    uint32_t  vsp;    // vertical sync polarity
    uint32_t  vblp;   // vertical blank polarity

    uint32_t  hav;    // horizontal active video
    uint32_t  hfp;    // horizontal front porch
    uint32_t  hsw;    // horizontal sync width
    uint32_t  hbp;    // horizontal back porch
    uint32_t  hsp;    // horizontal sync polarity
    uint32_t  hblp;   // horizontal blank polarity

    float     pclk;
};
typedef struct struct_vtc_timing_t vtc_timing_t;

struct struct_vtc_t
{
    uintptr_t         base_addr;
    XVtc              inst;
    XVtc_Signal       signal;       // parameters of video signal
    XVtc_Polarity     polarity;     // used polarity
    XVtc_SourceSelect src_sel;      // source selection setting
    uint32_t          locked;
    uint8_t           timing_sel;   // 0 - 1920x1080
};
typedef struct struct_vtc_t vtc_t;


uint32_t vtc_is_locked(XVtc *inst);
uint32_t vtc_get_status(XVtc *inst);
uint32_t vtc_get_error(XVtc *inst);
void vtc_reset(XVtc *inst);
void vtc_get_timing(vtc_t *vtc);
float vtc_get_fps( vtc_t *vtc, float pclk_mhz );
void vtc_fill_timing(vtc_t *vtc);
void vtc_set_timing(vtc_t *vtc);
void vtc_print_timing(vtc_t *vtc);

int vtc_init_common(vtc_t *vtc);
void vtc_init_gen(vtc_t *vtc);

int vtc_init(vtc_t *vtc, const char* dev_name);
int vtc_release(vtc_t *vtc);


#ifdef __cplusplus
}
#endif


#endif /* _VTC_HEADER_FILE_*/
