/********************************************
 * UTIA AV CR, v.v.i
 * Signal Processing Department
 * Pod Vodarenskou vezi 4
 * 182 08 Prague
 * Czech Republic
 * web: http://zs.utia.cas.cz
 * contact: sp@utia.cz
 *
 * Filename: adv7511_te.c
 * Date: 2023.01.04
 * Author: Lukas Kohout <kohoutl@utia.cas.cz>
 * Description:
 *    Trenz TE0701-05 HDMI OUT (ADV7511) interface
 *
 * Changelog:
 *
 *
 *
 *
 ********************************************/

#include <stdio.h>
#include <string.h>
#include <unistd.h>
// #include <sleep.h>

// #include <xiic.h>
// #include <xiic_l.h>

#include "adv7511_te.h"

#include "../i2c/i2c_te.h"


// ***************************************************************************
const adv7511_te_i2c_t adv7511_te_dvi_init_conf[] =
{

    {'w', 0xD6, 0xC0},  // HPD is always high

    {'w', 0x15, 0xF5}, // ID 5
    {'w', 0x16, 0x38}, // 8, style 1, falling

    {'w', 0x48, 0x08}, // align right, right justified, DDR low bits, normal bus order


    {'w', 0x55, 0x12}, // AVI AVI valid, RGB, PC
    {'w', 0x56, 0x28}, //
    {'w', 0x57, 0xC8}, // 08
    {'w', 0x59, 0x10}, //


    {'w', 0x17, 0x00}, // DE gen disabled, aspec 4:3

    {'w', 0xD0, 0x3C}, // Disable DDR Negative Edge CLK Delay
                       // delay = 0.0ns

//     {'w', 0xAF, 0x04}, // no HDCP
    {'w', 0xAF, 0x06}, // HDMI + HDCP

    // Conversion matrix IN=OUT
    //     {'w', 0x18, 0xA8},
    {'w', 0x18, 0x08},
    {'w', 0x19, 0x00},
    {'w', 0x1A, 0x00},
    {'w', 0x1B, 0x00},
    {'w', 0x1C, 0x00},
    {'w', 0x1D, 0x00},
    {'w', 0x1E, 0x00},
    {'w', 0x1F, 0x00},
    {'w', 0x20, 0x00},
    {'w', 0x21, 0x00},
    {'w', 0x22, 0x08},
    {'w', 0x23, 0x00},
    {'w', 0x24, 0x00},
    {'w', 0x25, 0x00},
    {'w', 0x26, 0x00},
    {'w', 0x27, 0x00},
    {'w', 0x28, 0x00},
    {'w', 0x29, 0x00},
    {'w', 0x2A, 0x00},
    {'w', 0x2B, 0x00},
    {'w', 0x2C, 0x08},
    {'w', 0x2D, 0x00},
    {'w', 0x2E, 0x00},
    {'w', 0x2F, 0x00},
    // ------------------------

    {'w', 0x41, 0x10},

    {'w', 0x98, 0x03}, // FIXED 0x03
    {'w', 0x9A, 0xE0}, // FIXED 0xE0
    {'w', 0x9C, 0x30}, // FIXED 0x30 PLL
    {'w', 0x9D, 0x61}, // xxxx_xx01 CLK Div
    {'w', 0xA2, 0xA4}, // FIXED
    {'w', 0xA3, 0xA4}, // FIXED

    {'w', 0xE0, 0xD0}, // FIXED
    {'w', 0xF9, 0x00},

    {'w', 0xFF, 0xFF},

    {'w', 0xBA, 0x60}, // no delay

    {0,0,0}
};



// ***************************************************************************



/*
 * configure ADV7511 chip
 */
int adv7511_te_init_sequence(int i2c_fd, unsigned char dev_addr, const adv7511_te_i2c_t conf[])
{
    unsigned int i = 0;
    int status;

    while(conf[i].rw) {
        switch(conf[i].rw) {
            case 'r':
                status = adv7511_te_i2c_read(i2c_fd, dev_addr, conf[i].reg);
                if (status < 0) return -1;
                break;
            case 'w':
                status = adv7511_te_i2c_write(i2c_fd, dev_addr, conf[i].reg, conf[i].data);
                if (status < 0) return -2;
                break;
            default:
                break;
        }
        i++;
    }
    return 0;
}


/*
 * adjust input clock delay (predefined values in its header)
 */
int adv7511_te_adjust_clkin_delay(int i2c_fd, unsigned char delay)
{
    int status;
    unsigned char val;

    // set input clock delay
    status = adv7511_te_i2c_read(i2c_fd, I2C_ADV7511_TE_ADDR, ADV7511_TE_CLK_DELAY_REG);
    if ( status < 0 ) {
        return -1;
    }

    val = (unsigned char)status & 0x1F;
    val |= delay;

    status = adv7511_te_i2c_write(i2c_fd, I2C_ADV7511_TE_ADDR, ADV7511_TE_CLK_DELAY_REG, val);
    if ( status < 0 ) {
        return -1;
    }

    // Adjust for the Input Video CLK Negative Edge for DDR Capture

    return 0;
}


/*
 * initialize ADV7511 HDMI
 */
int adv7511_te_init(int i2c_fd, unsigned char clkin_delay, unsigned char hpd_test)
{
    int status;
    int timeout;


    // clear HPD interrupt
    status = adv7511_te_i2c_write(i2c_fd, I2C_ADV7511_TE_ADDR, ADV7511_TE_EDID_INT_REG, ADV7511_TE_HPD_CLR);
    if (status < 0) {
        printf("Cannot clear ADV7511 HPD interrupt ... ");
        return -1;
    }


    /*
     * wait for HPD interrupt
     */
    if ( hpd_test ) {
        timeout = 100;
        status = 0;
        while ( ( status != ADV7511_TE_HPD ) && ( --timeout ) ) {
            status = adv7511_te_i2c_read(i2c_fd, I2C_ADV7511_TE_ADDR, ADV7511_TE_EDID_INT_REG) & ADV7511_TE_HPD;
            if ( status != ADV7511_TE_HPD ) usleep(10);
        }
        if ( !timeout ) {
            printf("HDMI cable is unplugged ... ");
            return -1;
        }
    }

    // configure ADV7511
    status = adv7511_te_init_sequence(i2c_fd, I2C_ADV7511_TE_ADDR, adv7511_te_dvi_init_conf);
    if (status != 0) {
        printf("Cannot configure ... ");
        return -1;
    }


    /*
     * adjust input clock delay (predefined values in its header)
     */
    status = adv7511_te_adjust_clkin_delay(i2c_fd, clkin_delay);
    if (status != 0) {
        printf("Cannot adjust inpt clock delay ... ");
        return -1;
    }

    // chip revision
    printf("rev. 0x%02X ... ", adv7511_te_i2c_read(i2c_fd, I2C_ADV7511_TE_ADDR, 0x00));


    return 0;
}


