/********************************************
 * UTIA AV CR, v.v.i
 * Signal Processing Department
 * Pod Vodarenskou vezi 4
 * 182 08 Prague
 * Czech Republic
 * web: http://zs.utia.cas.cz
 * contact: sp@utia.cz
 *
 * Filename: adv7511_te.h
 * Date: 2023.01.04
 * Author: Lukas Kohout <kohoutl@utia.cas.cz>
 * Description:
 *    Trenz TE0701-05 HDMI OUT (ADV7511) interface
 *
 * Changelog:
 *
 *
 *
 *
 ********************************************/

#ifndef _ADV7511_TE_HEADER_FILE_
#define _ADV7511_TE_HEADER_FILE_

#ifdef __cplusplus
extern "C" {
#endif



#define I2C_ADV7511_TE_ADDR                 0x39

#define ADV7511_TE_EDID_I2C_ADDR_REG        0x43
#define ADV7511_TE_EDID_INT_EN_REG          0x94
#define ADV7511_TE_EDID_INT_REG             0x96
#define ADV7511_TE_EDID_REREAD_REG          0xC9
#define ADV7511_TE_EDID_SEG_REG             0xC4

#define ADV7511_TE_EDID_INT_EN_MASK         0x04
#define ADV7511_TE_EDID_RDY_MASK            0x04
#define ADV7511_TE_EDID_REREAD_MASK         0x10
#define ADV7511_TE_EDID_HPD_INT_MASK        0x80
#define ADV7511_TE_EDID_HPD_MON_SENSE_MASK  0x40

#define ADV7511_TE_HPD                      (ADV7511_TE_EDID_HPD_INT_MASK | ADV7511_TE_EDID_HPD_MON_SENSE_MASK)
#define ADV7511_TE_HPD_CLR                  0x20

#define ADV7511_TE_HPD_TEST_ENABLED         1
#define ADV7511_TE_HPD_TEST_DISABLED        0

#define ADV7511_TE_CLK_DELAY_REG            0xBA
#define ADV7511_TE_CLK_DELAY_NEG_1_2_NS     0x00
#define ADV7511_TE_CLK_DELAY_NEG_0_8_NS     0x20
#define ADV7511_TE_CLK_DELAY_NEG_0_4_NS     0x40
#define ADV7511_TE_CLK_DELAY_0_0_NS         0x60
#define ADV7511_TE_CLK_DELAY_0_4_NS         0x80
#define ADV7511_TE_CLK_DELAY_0_8_NS         0xA0
#define ADV7511_TE_CLK_DELAY_1_2_NS         0xC0
#define ADV7511_TE_CLK_DELAY_1_6_NS         0xE0

typedef struct adv7511_te_i2c
{
    char          rw;
    unsigned char reg;
    unsigned char data;
} adv7511_te_i2c_t;


#include "../i2c/i2c_te.h"

#define adv7511_te_i2c_write  i2c_te_write
#define adv7511_te_i2c_read   i2c_te_read


int adv7511_te_init_sequence(int i2c_fd, unsigned char dev_addr, const adv7511_te_i2c_t conf[]);

int adv7511_te_i2c_init(int i2c_fd);
int adv7511_te_adjust_clkin_delay(int i2c_fd, unsigned char delay);
int adv7511_te_init(int i2c_fd, unsigned char clkin_delay, unsigned char hpd_test);

#ifdef __cplusplus
}
#endif

#endif
