/********************************************
 * UTIA AV CR, v.v.i
 * Signal Processing Department
 * Pod Vodarenskou vezi 4
 * 182 08 Prague
 * Czech Republic
 * web: http://zs.utia.cas.cz
 * contact: sp@utia.cz
 *
 * Filename: i2c_te.c
 * Date: 2023.01.04
 * Author: Lukas Kohout <kohoutl@utia.cas.cz>
 * Description:
 *    I2C controller init
 *
 *
 * Changelog:
 *
 *
 *
 *
 ********************************************/

#include <stdio.h>
#include <stdlib.h>
#include <unistd.h>
#include <fcntl.h>
#include <stdio.h>
#include <linux/i2c.h>
#include <linux/i2c-dev.h>
#include <sys/ioctl.h>

#include "i2c_te.h"


/*
 * I2C write register - 1B register address, 1B register Size
 */
int i2c_te_write(int i2c_fd, unsigned char dev_addr, unsigned char reg, unsigned char data)
{
    int           status;
    unsigned char buff[2];

    struct i2c_msg              msgs[1];
    struct i2c_rdwr_ioctl_data  msgset[1];

    buff[0] = reg;
    buff[1] = data;

    msgs[0].addr  = dev_addr;
    msgs[0].flags = 0;
    msgs[0].len   = 2;
    msgs[0].buf   = buff;

    msgset[0].msgs = msgs;
    msgset[0].nmsgs = 1;

    status = ioctl(i2c_fd, TE_I2C_RDWR, &msgset);
    if (status < 0) {
        return -1;
    }

    return 0;
}


/*
 * I2C read register - 1B register address, 1B register Size
 */
int i2c_te_read(int i2c_fd, unsigned char dev_addr, unsigned char reg)
{
    int           status;
    unsigned char buff_wr[1];
    unsigned char buff_rd[1];

    struct i2c_msg              msgs[2];
    struct i2c_rdwr_ioctl_data  msgset[1];

    buff_wr[0] = reg;

    msgs[0].addr = dev_addr;
    msgs[0].flags = 0;
    msgs[0].len = 1;
    msgs[0].buf = buff_wr;

    msgs[1].addr = dev_addr;
    msgs[1].flags = I2C_M_RD;
    msgs[1].len = 1;
    msgs[1].buf = buff_rd;

    msgset[0].msgs = msgs;
    msgset[0].nmsgs = 2;


    // set slave address
    status = ioctl(i2c_fd, TE_I2C_RDWR, &msgset);
    if (status < 0) {
        return -1;
    }

    return (int)*buff_rd;
}


/*
 * initialize I2C controller, On success it return I2C device FD
 */
int i2c_te_init(char *i2c_device)
{
    int status;
    int i2c_fd;


    /*
     * Open I2C device
     */
    i2c_fd = open(i2c_device, O_RDWR);
    if (i2c_fd < 0) {
        printf("Cannot open %s ... ", i2c_device);
        return -1;
    }

    return i2c_fd;
}


/*
 * Close I2C device, release FD
 */
void i2c_te_deinit(int *i2c_fd)
{
    int status;

    do {
        status = close(*i2c_fd);
    } while (status != -1);

    *i2c_fd = -1;
}


