/********************************************
 * UTIA AV CR, v.v.i
 * Signal Processing Department
 * Pod Vodarenskou vezi 4
 * 182 08 Prague
 * Czech Republic
 * web: http://zs.utia.cas.cz
 * contact: sp@utia.cz
 *
 * Filename: vdma.h
 * Date: 2022.03.08
 * Author: Lukas Kohout <kohoutl@utia.cas.cz>
 * Description:
 *   SW interface for Xilinx VDMA
 *
 * Changelog:
 *
 *
 *
 *
 ********************************************/

#ifndef __VDMA_H__
#define __VDMA_H__

#ifdef __cplusplus
extern "C" {
#endif

#include <stdint.h>
#include <assert.h>
#include <dirent.h>
#include <fcntl.h>
#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <sys/mman.h>
#include <unistd.h>
#include <stddef.h>

#include "xaxivdma.h"

//----------------------------------------------------------------
typedef struct {
  uint32_t x;
  uint32_t y;
} vdma_resolution_t;


typedef struct {
    // resolution = active video
    vdma_resolution_t res;

    // horizontal timing
    uint32_t          hfp;
    uint32_t          hsw;
    uint32_t          hbp;
    uint32_t          hsp;

    // vertical timing
    uint32_t          vfp;
    uint32_t          vsw;
    uint32_t          vbp;
    uint32_t          vsp;
} vdma_timing_t;


typedef struct {
    uint32_t x;
    uint32_t y;
} vdma_frame_offset_t;


struct struct_vdma_t {
    uintptr_t           core_base;

    uint16_t            dev_id;
    uintptr_t           fb_base;
    uint32_t            fb_num;

    XAxiVdma            inst;
    XAxiVdma_DmaSetup   tx_setup;
    XAxiVdma_DmaSetup   rx_setup;

    uint8_t             tx_pix_bytes;
    uint8_t             rx_pix_bytes;

    vdma_resolution_t   storage_res;

    vdma_timing_t       tx_timing;
    vdma_timing_t       rx_timing;

    vdma_frame_offset_t tx_offset;
    vdma_frame_offset_t rx_offset;
};
typedef struct struct_vdma_t vdma_t;


#define MIN_LINE_BYTES_WIDTH  10  // 2^10 = 1024 ~ 320*3 aligned to kB
#define MAX_LINE_BYTES_WIDTH  13  // 2^13 = 8192 ~ 1920*3 aligned to kB

#define MIN_STORAGE_BYTES_WIDTH  18  // 2^18 = 256 kB ~ (320*2)*240 aligned to kB


//----------------------------------------------------------------
int vdma_init(vdma_t *vdma, const char* dev_name);
int vdma_release(vdma_t *vdma);

// int vdma_init_common( vdma_t *vdma );
int vdma_init_common( vdma_t *vdma, XAxiVdma_Config *vdma_config );

int vdma_rx_start( XAxiVdma *vdma_inst );
int vdma_tx_start( XAxiVdma *vdma_inst );
int vdma_rx_stop( XAxiVdma *vdma_inst );
int vdma_tx_stop( XAxiVdma *vdma_inst );

uint32_t vdma_align_stride( uint32_t line_width, uint8_t byte_cnt );
uint32_t vdma_align_storage_size( uint32_t line_cnt, uint32_t stride );

int vdma_rx_init( vdma_t *vdma );
int vdma_tx_init( vdma_t *vdma );
void vdma_clr_errors( vdma_t *vdma );
int vdma_check_err( vdma_t *vdma );
void vdma_print_registers( vdma_t *vdma );

#ifdef __cplusplus
}
#endif

#endif // __VDMA_H__

